// @Name: ao_dc_voltage.cpp
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Ao functions of the USB - 5000 device, outputting fixed voltages.
// It sets the device's A0 - 3 outputs to 4.5V and 0V, and also sets A0 to 1V, A1 to 2V, A2 to 3V, and A3 to 4V.
// The analog voltage range is -10V ~ +10V.
// The Ao channels vary depending on the module, either Ao_4 or NONE.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5100(Ao_4):
// 0
// 0 All AOChannel is 4.5V
// 0 All AOChannel is 0V
// 0 All AOChannel is 4.5V
// 0 All AOChannel is 0V
// 0 All AOChannel is 4.5V
// 0 All AOChannel is 0V
// 0 AOChannel_0 is 1V
// 0 AOChannel_1 is 2V
// 0 AOChannel_2 is 3V
// 0 AOChannel_3 is 4V
// 0

#include <QCoreApplication>
#include <QDebug>
#include <QThread>
extern "C"
{
#include "USB5000.h"
}
int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    // Device index starts from zero
    int DevIndex = 0;

    // Ao channel, Ao channel 0 to Ao channel 3
    unsigned char AOChannel_0 = 0;
    unsigned char AOChannel_1 = 1;
    unsigned char AOChannel_2 = 2;
    unsigned char AOChannel_3 = 3;

    // voltage value, range is -10 to +10
    float voltage_4_5 = 4.5f;
    float voltage_4 = 4.0f;
    float voltage_3 = 3.0f;
    float voltage_2 = 2.0f;
    float voltage_1 = 1.0f;
    float voltage_0 = 0.0f;

    int error_code;

    // Open specific device and print return value.
    error_code = USB5OpenDevice(DevIndex);
    printf("%d\n", error_code);

    // Set all Ao channel 0V or 5V Interval of 1 second
    for (int channel = 0; channel < 4; channel++)
        error_code = SetUSB5AoImmediately(DevIndex, channel, voltage_4_5);

    printf("%d All AOChannel is 4.5V\n", error_code);
    QThread::sleep(3);

    for (int channel = 0; channel < 4; channel++)
        error_code = SetUSB5AoImmediately(DevIndex, channel, voltage_0);

    printf("%d All AOChannel is 0V\n", error_code);
    QThread::sleep(3);


    // Set single Ao channel
    error_code = SetUSB5AoImmediately(DevIndex, AOChannel_0, voltage_1);
    printf("%d AOChannel_0 is 1V\n", error_code);
    error_code = SetUSB5AoImmediately(DevIndex, AOChannel_1, voltage_2);
    printf("%d AOChannel_1 is 2V\n", error_code);
    error_code = SetUSB5AoImmediately(DevIndex, AOChannel_2, voltage_3);
    printf("%d AOChannel_2 is 3V\n", error_code);
    error_code = SetUSB5AoImmediately(DevIndex, AOChannel_3, voltage_4);
    printf("%d AOChannel_3 is 4V\n", error_code);

    // Close Device
    error_code = USB5CloseDevice(DevIndex);
    printf("%d\n", error_code);

    return 0;
}
