// @Name: do_immediately.cpp
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Do functions of the USB - 5000 device, performing digital output operations.
// It sets the Do output to HIGH/LOW, where 0V is LOW (default) and 3.3V is HIGH.
// The DO channels vary depending on the module, either Do_2 or Do_4 or Do_8.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5000(DO_4):
// 0
// 0
// Now Do3 - 0 is all HIGH
//...
// 0
// Now Do3 - 0 is all LOW
// 0
// Now DO 0 is 1
//...
// 0
// Now DO 0 is 0
// 0

#include <QCoreApplication>
#include <QDebug>
#include <QThread>
extern"C"{
#include "USB5000.h"
}
int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);
    // Device index starts from zero
    int DevIndex = 0;

    // For all select digital output channel:
    // 0xFF is select all digital output channel.
    unsigned int All_Do_Channel = 0xFF;

    // For all Do output Do_Value HIGH is 1 or LOW is 0:
    // Do_channel_4: 0~0xF(1111) Do3~Do0, Do_channel_8: 0~0xFF(1111 1111) Do7~Do0.
    unsigned int All_Do_Value = 0xFF;

    // For select single digital output channel:
    // Do_channel_4: 0~3 -> Do3~Do0, Do_channel_8: 0~7 -> Do7~Do0.
    unsigned int single_Do_Channel = 0;

    // single Do output Do_Value HIGH is 1:
    unsigned int single_Do_Value_HIGH = 1;

    // single Do output Do_Value LOW is 0:
    unsigned int single_Do_Value_LOW = 0;
    int error_code;

    // Open specific device and print return Do_Value.
    error_code = USB5OpenDevice(DevIndex);
    printf("%d\n", error_code);

    // Set All Do Pin output HIGH and then LOW, interval of 1 second.
    for (int i = 0; i < 3; ++i) {
        // Set Do Pin output, 15 = 0xF = 4-bit(1111) Do3 - 0 all HIGH.
        error_code = SetUSB5DoImmediately(DevIndex, All_Do_Channel, All_Do_Value);
        printf("%d\n", error_code);
        if (error_code == 0) {
            printf("Now Do7 - 0 is all HIGH\n");
        }
        // Necessary delay for easy observation
        QThread::sleep(1);
        // Set Do Pin output, 0 = 0x0 = 4-bit(0000) Do3 - 0 all LOW.
        error_code = SetUSB5DoImmediately(DevIndex, All_Do_Channel, 0);
        printf("%d\n", error_code);
        if (error_code == 0) {
            printf("Now Do7 - 0 is all LOW\n");
        }
        // Necessary delay for easy observation
        QThread::sleep(1);
    }

    // Set single DO_0 output HIGH/LOW, interval of 1 second.
    for (int i = 0; i < 3; ++i) {
        error_code = SetUSB5DoImmediately(DevIndex, single_Do_Channel, single_Do_Value_HIGH);
        printf("%d\n", error_code);
        if (error_code == 0) {
            printf("Now DO %d is %d\n", single_Do_Channel, single_Do_Value_HIGH);
        }
        QThread::sleep(1);
        error_code = SetUSB5DoImmediately(DevIndex, single_Do_Channel, single_Do_Value_LOW);
        printf("%d\n", error_code);
        if (error_code == 0) {
            printf("Now DO %d is %d\n", single_Do_Channel, single_Do_Value_LOW);
        }
        QThread::sleep(1);
    }

    // Close Device
    error_code = USB5CloseDevice(DevIndex);
    printf("%d\n", error_code);

    return 0;
}
